@@ -76,8 +76,10 @@ check_embedding_models<-function(object_list,
   #field to check
   #to_check<-c("model_name","model_date","model_method","model_version","model_language",
   #            "param_seq_length","param_chunks","param_overlap","param_aggregation")
-  to_check<-c("model_name","model_method","model_version","model_language",
-              "param_seq_length","param_chunks","param_overlap","param_aggregation")
+  #to_check<-c("model_name","model_method","model_version","model_language",
+  #            "param_seq_length","param_chunks","param_overlap","param_aggregation")
+  to_check<-c("model_name")
+
   tmp_model_config<-NULL
   tmp_results<-NULL
   for(i in 1:length(object_list)){
@@ -967,3 +969,65 @@ calc_standard_classification_measures<-function(true_values,predicted_values){
 
   return(results)
 }
+
+#'Clean pytorch log of transformers
+#'
+#'Function for preparing and cleaning the log created by an object of class Trainer
+#'from the python library 'transformer's
+#'
+#'@param log \code{data.frame} containing the log.
+#'
+#'@return Returns a \code{data.frame} containing epochs, loss, and val_loss.
+#'
+#'@family Auxiliary Functions
+#'@keywords internal
+#'
+#'@export
+clean_pytorch_log_transformers<-function(log){
+  max_epochs<-max(log$epoch)
+
+  cols=c("epoch","loss","val_loss")
+
+  cleaned_log<-matrix(data = NA,
+                      nrow = max_epochs,
+                      ncol = length(cols))
+  colnames(cleaned_log)=cols
+  for(i in 1:max_epochs){
+    cleaned_log[i,"epoch"]=i
+
+    tmp_loss=subset(log,log$epoch==i & is.na(log$loss)==FALSE)
+    tmp_loss=tmp_loss[1,"loss"]
+    cleaned_log[i,"loss"]=tmp_loss
+
+    tmp_val_loss=subset(log,log$epoch==i & is.na(log$eval_loss)==FALSE)
+    tmp_val_loss=tmp_val_loss[1,"eval_loss"]
+    cleaned_log[i,"val_loss"]=tmp_val_loss
+
+  }
+  return(as.data.frame(cleaned_log))
+}
+
+#'Check if NULL or NA
+#'
+#'Function for checking if an object is \code{NULL} or \code{NA}
+#'
+#'@param object An object to test.
+#'
+#'@return Returns \code{FALSE} if the object is not \code{NULL} and not \code{NA}.
+#'Returns \code{TRUE} in all other cases.
+#'
+#'@family Auxiliary Functions
+#'@keywords internal
+#'
+#'@export
+is.null_or_na<-function(object){
+  if(is.null(object)==FALSE){
+    if(anyNA(object)==FALSE){
+      return(FALSE)
+    } else {
+      return(TRUE)
+    }
+  } else {
+    return(TRUE)
+  }
+}
